@@ -22,12 +22,19 @@
 #include "config.h"
 
 #include <string.h>
+#include <math.h>
 
 #include "ev-annotation-window.h"
+#include "ev-color-contrast.h"
 #include "ev-stock-icons.h"
 #include "ev-view-marshal.h"
 #include "ev-document-misc.h"
 
+#if WITH_GSPELL
+#include <glib/gi18n.h>
+#include <gspell/gspell.h>
+#endif
+
 enum {
 	PROP_0,
 	PROP_ANNOTATION,
@@ -60,6 +67,11 @@ struct _EvAnnotationWindow {
 	gint          y;
 	gint          orig_x;
 	gint          orig_y;
+
+#if WITH_GSPELL
+	GspellTextView *spellcheck_view;
+	gboolean      enable_spellchecking;
+#endif
 };
 
 struct _EvAnnotationWindowClass {
@@ -75,8 +87,6 @@ static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE (EvAnnotationWindow, ev_annotation_window, GTK_TYPE_WINDOW)
 
-#define EV_ICON_SIZE_ANNOT_WINDOW (ev_annotation_window_get_icon_size())
-
 /* Cut and paste from gtkwindow.c */
 static void
 send_focus_change (GtkWidget *widget,
@@ -95,26 +105,6 @@ send_focus_change (GtkWidget *widget,
 	gdk_event_free (fevent);
 }
 
-static gdouble
-get_screen_dpi (EvAnnotationWindow *window)
-{
-	GdkScreen *screen;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (window));
-	return ev_document_misc_get_screen_dpi (screen);
-}
-
-static GtkIconSize
-ev_annotation_window_get_icon_size (void)
-{
-	static GtkIconSize icon_size = 0;
-
-	if (G_UNLIKELY (icon_size == 0))
-		icon_size = gtk_icon_size_register ("ev-icon-size-annot-window", 8, 8);
-
-	return icon_size;
-}
-
 static void
 ev_annotation_window_sync_contents (EvAnnotationWindow *window)
 {
@@ -132,33 +122,39 @@ ev_annotation_window_sync_contents (EvAnnotationWindow *window)
 
 static void
 ev_annotation_window_set_color (EvAnnotationWindow *window,
-				GdkColor           *color)
+				GdkRGBA            *color)
 {
-        GtkStyleProperties *properties;
-        GtkStyleProvider   *provider;
-	GdkRGBA             rgba;
+	GtkCssProvider     *css_provider = gtk_css_provider_new ();
+	g_autofree char    *rgba_str = gdk_rgba_to_string (color);
+	g_autofree char    *css_data = NULL;
+	g_autoptr (GError)  error = NULL;
+	g_autoptr (GdkRGBA) icon_color = ev_color_contrast_get_best_foreground_color (color);
+	g_autofree char    *icon_color_str = gdk_rgba_to_string (icon_color);
+	css_data = g_strdup_printf ("button {border-color: %1$s; color: %2$s; -gtk-icon-shadow:0 0; box-shadow:0 0;}\n"
+				    "button:hover {background: lighter(%1$s); border-color: darker(%1$s);}\n"
+				    "button:active {background: darker(%1$s);}\n"
+				    "evannotationwindow.background { color: %2$s; }\n"
+				    "evannotationwindow.background:backdrop { color: alpha(%2$s, .75); }\n"
+				    "evannotationwindow.background, button {background: %1$s}",
+				    rgba_str, icon_color_str);
 
-        rgba.red = color->red / 65535.;
-        rgba.green = color->green / 65535.;
-        rgba.blue = color->blue / 65535.;
-        rgba.alpha = 1;
+	gtk_css_provider_load_from_data (css_provider, css_data, strlen (css_data), &error);
+	if (error != NULL)
+		g_error ("%s", error->message);
 
-        properties = gtk_style_properties_new ();
-        gtk_style_properties_set (properties, 0,
-                                  "color", &rgba,
-                                  "background-color", &rgba,
-                                  NULL);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (window)),
+					GTK_STYLE_PROVIDER (css_provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (window->close_button),
+					GTK_STYLE_PROVIDER (css_provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gtk_style_context_add_class (gtk_widget_get_style_context (window->close_button), "circular");
+}
 
-        provider = GTK_STYLE_PROVIDER (properties);
-        gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (window)),
-                                        provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        gtk_style_context_add_provider (gtk_widget_get_style_context (window->close_button),
-                                        provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        gtk_style_context_add_provider (gtk_widget_get_style_context (window->resize_se),
-                                        provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        gtk_style_context_add_provider (gtk_widget_get_style_context (window->resize_sw),
-                                        provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        g_object_unref (properties);
+static void
+ev_annotation_window_set_opacity (EvAnnotationWindow *window,
+		                  gdouble             opacity)
+{
+	gtk_widget_set_opacity (GTK_WIDGET (window), opacity);
+	gtk_widget_set_opacity (GTK_WIDGET (window->text_view), opacity);
 }
 
 static void
@@ -177,10 +173,21 @@ ev_annotation_window_color_changed (EvAnnotation       *annot,
 				    GParamSpec         *pspec,
 				    EvAnnotationWindow *window)
 {
-	GdkColor color;
+	GdkRGBA rgba;
 
-	ev_annotation_get_color (annot, &color);
-	ev_annotation_window_set_color (window, &color);
+	ev_annotation_get_rgba (annot, &rgba);
+	ev_annotation_window_set_color (window, &rgba);
+}
+
+static void
+ev_annotation_window_opacity_changed (EvAnnotation       *annot,
+		                      GParamSpec         *pspec,
+				      EvAnnotationWindow *window)
+{
+	gdouble opacity;
+
+	opacity = ev_annotation_markup_get_opacity (EV_ANNOTATION_MARKUP (annot));
+	ev_annotation_window_set_opacity (window, opacity);
 }
 
 static void
@@ -190,8 +197,7 @@ ev_annotation_window_dispose (GObject *object)
 
 	if (window->annotation) {
 		ev_annotation_window_sync_contents (window);
-		g_object_unref (window->annotation);
-		window->annotation = NULL;
+		g_clear_object (&window->annotation);
 	}
 
 	(* G_OBJECT_CLASS (ev_annotation_window_parent_class)->dispose) (object);
@@ -253,20 +259,20 @@ ev_annotation_window_set_resize_cursor (GtkWidget          *widget,
 						     GDK_BOTTOM_LEFT_CORNER :
 						     GDK_BOTTOM_RIGHT_CORNER);
 		gdk_window_set_cursor (gdk_window, cursor);
-		gdk_cursor_unref (cursor);
+		g_object_unref (cursor);
 	} else {
 		gdk_window_set_cursor (gdk_window, NULL);
 	}
 }
 
-static gboolean
-text_view_button_press (GtkWidget          *widget,
-			GdkEventButton     *event,
-			EvAnnotationWindow *window)
+static void
+text_view_state_flags_changed (GtkWidget     *widget,
+			       GtkStateFlags  previous_flags)
 {
-	ev_annotation_window_grab_focus (window);
+	GtkStateFlags current_flags = gtk_widget_get_state_flags (widget);
 
-	return FALSE;
+	if (current_flags & GTK_STATE_FLAG_BACKDROP)
+		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (widget), FALSE);
 }
 
 static void
@@ -276,176 +282,6 @@ ev_annotation_window_close (EvAnnotationWindow *window)
 	g_signal_emit (window, signals[CLOSED], 0);
 }
 
-static void
-ev_annotation_window_init (EvAnnotationWindow *window)
-{
-	GtkWidget *vbox, *hbox;
-	GtkWidget *icon;
-	GtkWidget *swindow;
-
-	gtk_widget_set_can_focus (GTK_WIDGET (window), TRUE);
-
-	vbox = gtk_vbox_new (FALSE, 0);
-
-	/* Title bar */
-	hbox = gtk_hbox_new (FALSE, 0);
-
-	icon = gtk_image_new (); /* FIXME: use the annot icon */
-	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
-	gtk_widget_show (icon);
-
-	window->title = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), window->title, TRUE, TRUE, 0);
-	gtk_widget_show (window->title);
-
-	window->close_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (window->close_button), GTK_RELIEF_NONE);
-	gtk_container_set_border_width (GTK_CONTAINER (window->close_button), 0);
-	g_signal_connect_swapped (window->close_button, "clicked",
-				  G_CALLBACK (ev_annotation_window_close),
-				  window);
-	icon = gtk_image_new_from_stock (EV_STOCK_CLOSE, EV_ICON_SIZE_ANNOT_WINDOW);
-	gtk_container_add (GTK_CONTAINER (window->close_button), icon);
-	gtk_widget_show (icon);
-
-	gtk_box_pack_start (GTK_BOX (hbox), window->close_button, FALSE, FALSE, 0);
-	gtk_widget_show (window->close_button);
-
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show (hbox);
-
-	/* Contents */
-	swindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	window->text_view = gtk_text_view_new ();
-	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (window->text_view), GTK_WRAP_WORD);
-	g_signal_connect (window->text_view, "button_press_event",
-			  G_CALLBACK (text_view_button_press),
-			  window);
-	gtk_container_add (GTK_CONTAINER (swindow), window->text_view);
-	gtk_widget_show (window->text_view);
-
-	gtk_box_pack_start (GTK_BOX (vbox), swindow, TRUE, TRUE, 0);
-	gtk_widget_show (swindow);
-
-	/* Resize bar */
-	gtk_window_set_has_resize_grip (GTK_WINDOW(window), FALSE);
-	hbox = gtk_hbox_new (FALSE, 0);
-
-	window->resize_sw = gtk_event_box_new ();
-	gtk_widget_add_events (window->resize_sw, GDK_BUTTON_PRESS_MASK);
-	g_signal_connect_swapped (window->resize_sw, "button-press-event",
-				  G_CALLBACK (ev_annotation_window_resize),
-				  window);
-	g_signal_connect (window->resize_sw, "realize",
-			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
-			  window);
-
-	icon = gtk_image_new_from_stock (EV_STOCK_RESIZE_SW, EV_ICON_SIZE_ANNOT_WINDOW);
-	gtk_container_add (GTK_CONTAINER (window->resize_sw), icon);
-	gtk_widget_show (icon);
-	gtk_box_pack_start (GTK_BOX (hbox), window->resize_sw, FALSE, FALSE, 0);
-	gtk_widget_show (window->resize_sw);
-
-	window->resize_se = gtk_event_box_new ();
-	gtk_widget_add_events (window->resize_se, GDK_BUTTON_PRESS_MASK);
-	g_signal_connect_swapped (window->resize_se, "button-press-event",
-				  G_CALLBACK (ev_annotation_window_resize),
-				  window);
-	g_signal_connect (window->resize_se, "realize",
-			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
-			  window);
-
-	icon = gtk_image_new_from_stock (EV_STOCK_RESIZE_SE, EV_ICON_SIZE_ANNOT_WINDOW);
-	gtk_container_add (GTK_CONTAINER (window->resize_se), icon);
-	gtk_widget_show (icon);
-	gtk_box_pack_end (GTK_BOX (hbox), window->resize_se, FALSE, FALSE, 0);
-	gtk_widget_show (window->resize_se);
-
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show (hbox);
-
-	gtk_container_add (GTK_CONTAINER (window), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (window), 0);
-	gtk_widget_show (vbox);
-
-	gtk_widget_add_events (GTK_WIDGET (window),
-			       GDK_BUTTON_PRESS_MASK |
-			       GDK_KEY_PRESS_MASK);
-	gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
-
-	gtk_container_set_border_width (GTK_CONTAINER (window), 2);
-
-	gtk_window_set_accept_focus (GTK_WINDOW (window), TRUE);
-	gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
-	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), TRUE);
-	gtk_window_set_skip_pager_hint (GTK_WINDOW (window), TRUE);
-	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
-}
-
-static GObject *
-ev_annotation_window_constructor (GType                  type,
-				  guint                  n_construct_properties,
-				  GObjectConstructParam *construct_params)
-{
-	GObject            *object;
-	EvAnnotationWindow *window;
-	EvAnnotation       *annot;
-	EvAnnotationMarkup *markup;
-	const gchar        *contents;
-	const gchar        *label;
-	GdkColor            color;
-	EvRectangle        *rect;
-	gdouble             scale;
-
-	object = G_OBJECT_CLASS (ev_annotation_window_parent_class)->constructor (type,
-										  n_construct_properties,
-										  construct_params);
-	window = EV_ANNOTATION_WINDOW (object);
-	annot = window->annotation;
-	markup = EV_ANNOTATION_MARKUP (annot);
-
-	gtk_window_set_transient_for (GTK_WINDOW (window), window->parent);
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (window), FALSE);
-
-	label = ev_annotation_markup_get_label (markup);
-	window->is_open = ev_annotation_markup_get_popup_is_open (markup);
-	ev_annotation_markup_get_rectangle (markup, &window->rect);
-
-	rect = &window->rect;
-
-	/* Rectangle is at doc resolution (72.0) */
-	scale = get_screen_dpi (window) / 72.0;
-	gtk_window_resize (GTK_WINDOW (window),
-			   (gint)((rect->x2 - rect->x1) * scale),
-			   (gint)((rect->y2 - rect->y1) * scale));
-
-	ev_annotation_get_color (annot, &color);
-	ev_annotation_window_set_color (window, &color);
-	gtk_widget_set_name (GTK_WIDGET (window), ev_annotation_get_name (annot));
-	gtk_window_set_title (GTK_WINDOW (window), label);
-	gtk_label_set_text (GTK_LABEL (window->title), label);
-
-	contents = ev_annotation_get_contents (annot);
-	if (contents) {
-		GtkTextBuffer *buffer;
-
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (window->text_view));
-		gtk_text_buffer_set_text (buffer, contents, -1);
-	}
-
-	g_signal_connect (annot, "notify::label",
-			  G_CALLBACK (ev_annotation_window_label_changed),
-			  window);
-	g_signal_connect (annot, "notify::color",
-			  G_CALLBACK (ev_annotation_window_color_changed),
-			  window);
-
-	return object;
-}
-
 static gboolean
 ev_annotation_window_button_press_event (GtkWidget      *widget,
 					 GdkEventButton *event)
@@ -467,6 +303,203 @@ ev_annotation_window_button_press_event (GtkWidget      *widget,
 	return FALSE;
 }
 
+static void
+ev_annotation_window_init (EvAnnotationWindow *window)
+{
+	GtkWidget    *vbox, *hbox;
+	GtkWidget    *icon;
+	GtkWidget    *swindow;
+	GtkWidget    *header;
+	GtkIconTheme *icon_theme;
+	GdkPixbuf    *pixbuf;
+
+	icon_theme = gtk_icon_theme_get_default ();
+
+	gtk_widget_set_can_focus (GTK_WIDGET (window), TRUE);
+
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+
+	/* Title bar */
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+
+	icon = gtk_image_new (); /* FIXME: use the annot icon */
+	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+	gtk_widget_show (icon);
+
+	header = gtk_event_box_new ();
+	gtk_widget_add_events (header, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect_swapped (header, "button-press-event",
+				  G_CALLBACK (ev_annotation_window_button_press_event),
+			          window);
+
+	window->title = gtk_label_new (NULL);
+	gtk_container_add (GTK_CONTAINER (header), window->title);
+	gtk_widget_show (window->title);
+
+	gtk_box_pack_start (GTK_BOX (hbox), header, TRUE, TRUE, 0);
+	gtk_widget_show (header);
+
+	window->close_button = gtk_button_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_BUTTON);
+	g_signal_connect_swapped (window->close_button, "clicked",
+				  G_CALLBACK (ev_annotation_window_close),
+				  window);
+
+	gtk_box_pack_start (GTK_BOX (hbox), window->close_button, FALSE, FALSE, 0);
+	gtk_widget_show (window->close_button);
+
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+
+	/* Contents */
+	swindow = gtk_scrolled_window_new (NULL, NULL);
+	window->text_view = gtk_text_view_new ();
+
+#if WITH_GSPELL
+	window->spellcheck_view = NULL;
+	window->spellcheck_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (window->text_view));
+	gspell_text_view_basic_setup (window->spellcheck_view);
+#endif
+
+	gtk_container_set_border_width (GTK_CONTAINER (window->text_view), 6);
+	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (window->text_view), GTK_WRAP_WORD);
+	g_signal_connect (window->text_view, "state-flags-changed",
+			  G_CALLBACK (text_view_state_flags_changed),
+			  window);
+	gtk_container_add (GTK_CONTAINER (swindow), window->text_view);
+	gtk_widget_show (window->text_view);
+
+	gtk_box_pack_start (GTK_BOX (vbox), swindow, TRUE, TRUE, 0);
+	gtk_widget_show (swindow);
+
+	/* Resize bar */
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+
+	window->resize_sw = gtk_event_box_new ();
+	gtk_widget_add_events (window->resize_sw, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect_swapped (window->resize_sw, "button-press-event",
+				  G_CALLBACK (ev_annotation_window_resize),
+				  window);
+	g_signal_connect (window->resize_sw, "realize",
+			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
+			  window);
+
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, EV_STOCK_RESIZE_SW, 8,
+					   GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+	icon = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
+	gtk_container_add (GTK_CONTAINER (window->resize_sw), icon);
+	gtk_widget_show (icon);
+	gtk_box_pack_start (GTK_BOX (hbox), window->resize_sw, FALSE, FALSE, 0);
+	gtk_widget_show (window->resize_sw);
+
+	window->resize_se = gtk_event_box_new ();
+	gtk_widget_add_events (window->resize_se, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect_swapped (window->resize_se, "button-press-event",
+				  G_CALLBACK (ev_annotation_window_resize),
+				  window);
+	g_signal_connect (window->resize_se, "realize",
+			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
+			  window);
+
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, EV_STOCK_RESIZE_SE, 8,
+					   GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+	icon = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
+	gtk_container_add (GTK_CONTAINER (window->resize_se), icon);
+	gtk_widget_show (icon);
+	gtk_box_pack_end (GTK_BOX (hbox), window->resize_se, FALSE, FALSE, 0);
+	gtk_widget_show (window->resize_se);
+
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+
+	gtk_container_add (GTK_CONTAINER (window), vbox);
+	gtk_widget_show (vbox);
+
+	gtk_widget_add_events (GTK_WIDGET (window),
+			       GDK_BUTTON_PRESS_MASK |
+			       GDK_KEY_PRESS_MASK);
+
+	gtk_container_set_border_width (GTK_CONTAINER (window), 2);
+
+	gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), TRUE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (window), TRUE);
+	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
+}
+
+static GObject *
+ev_annotation_window_constructor (GType                  type,
+				  guint                  n_construct_properties,
+				  GObjectConstructParam *construct_params)
+{
+	GObject            *object;
+	EvAnnotationWindow *window;
+	EvAnnotation       *annot;
+	EvAnnotationMarkup *markup;
+	const gchar        *contents;
+	const gchar        *label;
+	GdkRGBA             color;
+	EvRectangle        *rect;
+	gdouble             scale;
+	gdouble             opacity;
+
+	object = G_OBJECT_CLASS (ev_annotation_window_parent_class)->constructor (type,
+										  n_construct_properties,
+										  construct_params);
+	window = EV_ANNOTATION_WINDOW (object);
+	annot = window->annotation;
+	markup = EV_ANNOTATION_MARKUP (annot);
+
+	gtk_window_set_transient_for (GTK_WINDOW (window), window->parent);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (window), FALSE);
+
+	label = ev_annotation_markup_get_label (markup);
+	window->is_open = ev_annotation_markup_get_popup_is_open (markup);
+	ev_annotation_markup_get_rectangle (markup, &window->rect);
+
+	rect = &window->rect;
+
+	/* Rectangle is at doc resolution (72.0) */
+	scale = ev_document_misc_get_widget_dpi (GTK_WIDGET (window)) / 72.0;
+	gtk_window_resize (GTK_WINDOW (window),
+			   (gint)((rect->x2 - rect->x1) * scale),
+			   (gint)((rect->y2 - rect->y1) * scale));
+
+	ev_annotation_get_rgba (annot, &color);
+	ev_annotation_window_set_color (window, &color);
+
+	opacity = ev_annotation_markup_get_opacity (markup);
+	ev_annotation_window_set_opacity (window, opacity);
+
+	gtk_widget_set_name (GTK_WIDGET (window), ev_annotation_get_name (annot));
+	gtk_window_set_title (GTK_WINDOW (window), label);
+	gtk_label_set_text (GTK_LABEL (window->title), label);
+
+	contents = ev_annotation_get_contents (annot);
+	if (contents) {
+		GtkTextBuffer *buffer;
+
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (window->text_view));
+		gtk_text_buffer_set_text (buffer, contents, -1);
+	}
+
+	g_signal_connect (annot, "notify::label",
+			  G_CALLBACK (ev_annotation_window_label_changed),
+			  window);
+	g_signal_connect (annot, "notify::rgba",
+			  G_CALLBACK (ev_annotation_window_color_changed),
+			  window);
+	g_signal_connect (annot, "notify::opacity",
+			  G_CALLBACK (ev_annotation_window_opacity_changed),
+			  window);
+
+#if WITH_GSPELL
+        gspell_text_view_set_inline_spell_checking (window->spellcheck_view, ev_annotation_window_get_enable_spellchecking (window));
+#endif
+	return object;
+}
+
 static gboolean
 ev_annotation_window_configure_event (GtkWidget         *widget,
 				      GdkEventConfigure *event)
@@ -497,6 +530,10 @@ ev_annotation_window_focus_in_event (GtkWidget     *widget,
 		window->in_move = FALSE;
 	}
 
+	gtk_widget_grab_focus (window->text_view);
+	send_focus_change (window->text_view, TRUE);
+	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (window->text_view), TRUE);
+
 	return FALSE;
 }
 
@@ -511,6 +548,18 @@ ev_annotation_window_focus_out_event (GtkWidget     *widget,
 	return FALSE;
 }
 
+static gboolean
+ev_annotation_window_key_press_event (GtkWidget   *widget,
+                                      GdkEventKey *event)
+{
+        if (event->keyval == GDK_KEY_Escape) {
+                ev_annotation_window_close (EV_ANNOTATION_WINDOW (widget));
+                return TRUE;
+        }
+
+        return GTK_WIDGET_CLASS (ev_annotation_window_parent_class)->key_press_event (widget, event);
+}
+
 static void
 ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 {
@@ -521,11 +570,12 @@ ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 	g_object_class->set_property = ev_annotation_window_set_property;
 	g_object_class->dispose = ev_annotation_window_dispose;
 
-	gtk_widget_class->button_press_event = ev_annotation_window_button_press_event;
 	gtk_widget_class->configure_event = ev_annotation_window_configure_event;
 	gtk_widget_class->focus_in_event = ev_annotation_window_focus_in_event;
 	gtk_widget_class->focus_out_event = ev_annotation_window_focus_out_event;
+        gtk_widget_class->key_press_event = ev_annotation_window_key_press_event;
 
+	gtk_widget_class_set_css_name (gtk_widget_class, "evannotationwindow");
 	g_object_class_install_property (g_object_class,
 					 PROP_ANNOTATION,
 					 g_param_spec_object ("annotation",
@@ -533,7 +583,8 @@ ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 							      "The annotation associated to the window",
 							      EV_TYPE_ANNOTATION_MARKUP,
 							      G_PARAM_WRITABLE |
-							      G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_CONSTRUCT_ONLY |
+                                                              G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (g_object_class,
 					 PROP_PARENT,
 					 g_param_spec_object ("parent",
@@ -541,7 +592,8 @@ ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 							      "The parent window",
 							      GTK_TYPE_WINDOW,
 							      G_PARAM_WRITABLE |
-							      G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_CONSTRUCT_ONLY |
+                                                              G_PARAM_STATIC_STRINGS));
 	signals[CLOSED] =
 		g_signal_new ("closed",
 			      G_TYPE_FROM_CLASS (g_object_class),
@@ -652,3 +704,29 @@ ev_annotation_window_ungrab_focus (EvAnnotationWindow *window)
 
 	ev_annotation_window_sync_contents (window);
 }
+
+void
+ev_annotation_window_set_enable_spellchecking (EvAnnotationWindow *window,
+                                               gboolean enable_spellchecking)
+{
+        g_return_if_fail (EV_IS_ANNOTATION_WINDOW (window));
+
+#if WITH_GSPELL
+        if (enable_spellchecking == ev_annotation_window_get_enable_spellchecking (window))
+                return;
+
+        window->enable_spellchecking = enable_spellchecking;
+        gspell_text_view_set_inline_spell_checking (window->spellcheck_view, enable_spellchecking);
+#endif
+}
+
+gboolean
+ev_annotation_window_get_enable_spellchecking (EvAnnotationWindow *window)
+{
+        g_return_val_if_fail (EV_IS_ANNOTATION_WINDOW (window), FALSE);
+#if WITH_GSPELL
+        return window->enable_spellchecking;
+#else
+        return FALSE;
+#endif
+}
